/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "$(appName)-ref.h"
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <storage.h>
#include <thumbnail_util.h>

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *layout;
	Evas_Object *nf;
} appdata_s;

Evas_Object *t_path;
Evas_Object *t_width;
Evas_Object *t_height;
Evas_Object *t_result;
Evas_Object *thumbnail;
Evas_Object *cbox;

/*
 * @brief Function will be operated when window deletion is requested
 * @param[in] data The data to be passed to the callback function
 * @param[in] obj The Evas object handle to be passed to the callback function
 * @param[in] event_info The system event information
 */
static void win_delete_request_cb(void *data , Evas_Object *obj , void *event_info)
{
	ui_app_exit();
}

/*
 * @brief Callback to get the path of the root from storage api
 * @param[in] storage_id  The storage id information
 * @param[in] type  The type of a storage
 * @param[in] state  The state of a storage
 * @param[in] path  The path of a storage
 * @param[in] user_data  The user data
 */
static bool _storage_cb(int storage_id, storage_type_e type, storage_state_e state, const char *path, void *user_data)
{
	if (STORAGE_TYPE_INTERNAL == type) {
		int *internal_storage_id = (int *)user_data;
		if (NULL != internal_storage_id)
			*internal_storage_id = storage_id;
		return false;
	} else {
		return true;
	}
}

/*
 * @brief Function to pop the last view
 * @param[in] data The user data
 * @param[in] it The evas object item for gui object
 */
static Eina_Bool nf_it_pop_cb(void *data, Elm_Object_Item *it)
{
	appdata_s *ad = data;
	/* Don't pop the last view but hide window */
	elm_win_lower(ad->win);
	return EINA_FALSE;
}

/*
 * @brief Function to generate thumbnail when gui object is clicked
 * @param[in] data The user data
 * @param[in] obj The evas object for gui object
 * @param[in] event_info  The system event information
 */
static void gen_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	char *parent_path = NULL;
	char full_path[100] = {0, };
	char result_path[100] = {0, };
	int width;
	int height;
	int ret = THUMBNAIL_UTIL_ERROR_NONE;
	int internal_storage_id = -1;

	ret = storage_foreach_device_supported(_storage_cb, &internal_storage_id);
	 if (STORAGE_ERROR_NONE != ret) {
		 dlog_print(DLOG_ERROR, LOG_TAG, "storage_foreach_device_supported() failed! Error code = %d", ret);
		 return;
	 }

	ret = storage_get_root_directory(internal_storage_id, &parent_path);
	if (STORAGE_ERROR_NONE != ret) {
		dlog_print(DLOG_ERROR, LOG_TAG, "storage_get_root_directory() failed! Error code = %d", ret);
		return;
	}

	snprintf(full_path, sizeof(full_path), "%s/%s", parent_path, elm_entry_entry_get(t_path));
	snprintf(result_path, sizeof(result_path), "%s/%s", parent_path, elm_entry_entry_get(t_result));

	width = atoi(elm_entry_entry_get(t_width));
	height = atoi(elm_entry_entry_get(t_height));

	ret = thumbnail_util_extract_to_file(full_path, width, height, result_path);
	if (THUMBNAIL_UTIL_ERROR_NONE != ret) {
		dlog_print(DLOG_ERROR, LOG_TAG, "thumbnail_util_extract_to_file() failed! Error code = %d", ret);
		return;
	}

	if (thumbnail == NULL) {
		thumbnail = elm_image_add(cbox);
		evas_object_size_hint_align_set(thumbnail, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_size_hint_weight_set(thumbnail, 0.7, EVAS_HINT_EXPAND);
	}

	elm_image_file_set(thumbnail, result_path, NULL);
	evas_object_show(thumbnail);

	if (parent_path) {
		free(parent_path);
		parent_path = NULL;
	}
}

/*
 * @brief Function to create box gui structure
 * @param[in] ad The data structure to manage gui object
 */
static Evas_Object* create_box(appdata_s *ad)
{
	Evas_Object *box;


	box = elm_box_add(ad->nf);
	elm_box_padding_set(box, 30, 30);
	elm_object_content_set(ad->nf, box);
	elm_box_horizontal_set(box, EINA_FALSE);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box);

	Evas_Object *abox = elm_box_add(box);
	Evas_Object *bbox = elm_box_add(box);
	cbox = elm_box_add(box);
	/* Create a box for adding content */
	elm_box_padding_set(abox, 10, 10);
	elm_box_horizontal_set(abox, EINA_FALSE);
	evas_object_size_hint_align_set(abox, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(abox, EVAS_HINT_EXPAND, 0.1);
	evas_object_show(abox);
	/* Create sub box in abox */
	Evas_Object *box_path = elm_box_add(abox);
	elm_box_horizontal_set(box_path, EINA_TRUE);
	evas_object_size_hint_align_set(box_path, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box_path, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box_path);
	/* Insert item */
	Evas_Object *l_path = elm_label_add(box_path);
	elm_object_text_set(l_path, "<font_size=30><b>Path</b></font/>");
	evas_object_size_hint_align_set(l_path, 0.5, 0.5);
	evas_object_size_hint_weight_set(l_path, 0.3, EVAS_HINT_EXPAND);
	evas_object_show(l_path);

	t_path = elm_entry_add(box_path);
	char *path = "sample.jpg";
	elm_object_text_set(t_path, path);
	evas_object_size_hint_align_set(t_path, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(t_path, 0.7, EVAS_HINT_EXPAND);
	evas_object_show(t_path);

	Evas_Object *box_width = elm_box_add(abox);
	elm_box_horizontal_set(box_width, EINA_TRUE);
	evas_object_size_hint_align_set(box_width, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box_width, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box_width);
	/* Insert item */
	Evas_Object *l_width = elm_label_add(box_width);
	elm_object_text_set(l_width, "<font_size=30><b>Width</b></font/>");
	evas_object_size_hint_align_set(l_width, 0.5, 0.5);
	evas_object_size_hint_weight_set(l_width, 0.3, EVAS_HINT_EXPAND);
	evas_object_show(l_width);

	t_width = elm_entry_add(box_width);
	elm_object_text_set(t_width, "512");
	evas_object_size_hint_align_set(t_width, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(t_width, 0.7, EVAS_HINT_EXPAND);
	evas_object_show(t_width);

	Evas_Object *box_height = elm_box_add(abox);
	elm_box_horizontal_set(box_height, EINA_TRUE);
	evas_object_size_hint_align_set(box_height, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box_height, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box_height);

	/* Insert item */
	Evas_Object *l_height = elm_label_add(box_height);
	elm_object_text_set(l_height, "<font_size=30><b>Height</b></font/>");
	evas_object_size_hint_align_set(l_height, 0.5, 0.5);
	evas_object_size_hint_weight_set(l_height, 0.3, EVAS_HINT_EXPAND);
	evas_object_show(l_height);

	t_height = elm_entry_add(box_height);
	elm_object_text_set(t_height, "288");
	evas_object_size_hint_align_set(t_height, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(t_height, 0.7, EVAS_HINT_EXPAND);
	evas_object_show(t_height);

	Evas_Object *box_result = elm_box_add(abox);
	elm_box_horizontal_set(box_result, EINA_TRUE);
	evas_object_size_hint_align_set(box_result, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box_result, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(box_result);
	/* Insert item */
	Evas_Object *l_result = elm_label_add(box_path);
	elm_object_text_set(l_result, "<font_size=30><b>Result</b></font/>");
	evas_object_size_hint_align_set(l_result, 0.5, 0.5);
	evas_object_size_hint_weight_set(l_result, 0.3, EVAS_HINT_EXPAND);
	evas_object_show(l_result);

	t_result = elm_entry_add(box_path);
	char *result = "out.jpg";
	elm_object_text_set(t_result, result);
	evas_object_size_hint_align_set(t_result, EVAS_HINT_FILL, 0.5);
	evas_object_size_hint_weight_set(t_result, 0.7, EVAS_HINT_EXPAND);
	evas_object_show(t_result);
	/* Create a box for adding content */
	elm_box_padding_set(cbox, 50, 50);
	elm_box_horizontal_set(cbox, EINA_TRUE);
	evas_object_size_hint_align_set(cbox, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(cbox, EVAS_HINT_EXPAND, 0.8);
	evas_object_show(cbox);
	/* Insert item */
	thumbnail = elm_image_add(cbox);
	evas_object_size_hint_align_set(thumbnail, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(thumbnail, EVAS_HINT_EXPAND, 1.0);
	evas_object_show(thumbnail);

	/* Create a box for adding content */
	elm_box_padding_set(bbox, 10, 10);
	evas_object_size_hint_align_set(bbox, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(bbox, EVAS_HINT_EXPAND, 0.1);
	evas_object_show(bbox);

	/* Create "Generate" button */
	Evas_Object *bt_gen = elm_button_add(bbox);
	elm_object_text_set(bt_gen, "<font_size=40><b>Generate</b></font/>");
	elm_box_horizontal_set(bt_gen, EINA_FALSE);
	evas_object_size_hint_align_set(bt_gen, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(bt_gen, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_smart_callback_add(bt_gen, "clicked", gen_clicked_cb, NULL);
	evas_object_show(bt_gen);

	elm_box_pack_end(box, abox);
	elm_box_pack_end(abox, box_path);
	elm_box_pack_end(abox, box_width);
	elm_box_pack_end(abox, box_height);
	elm_box_pack_end(abox, box_result);
	elm_box_pack_end(box_path, l_path);
	elm_box_pack_end(box_path, t_path);
	elm_box_pack_end(box_width, l_width);
	elm_box_pack_end(box_width, t_width);
	elm_box_pack_end(box_height, l_height);
	elm_box_pack_end(box_height, t_height);
	elm_box_pack_end(box_result, l_result);
	elm_box_pack_end(box_result, t_result);

	elm_box_pack_end(box, bbox);
	elm_box_pack_end(box, cbox);
	elm_box_pack_end(bbox, bt_gen);

	elm_box_pack_end(cbox, thumbnail);
	return box;
}

/*
 * @brief Function to create naviframe gui structure
 * @param[in] ad The data structure to manage gui object
 */
static Evas_Object* create_main_view(appdata_s *ad)
{
	Elm_Object_Item *nf_it;
	Evas_Object *box;

	ad->nf = elm_naviframe_add(ad->conform);
	eext_object_event_callback_add(ad->nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, ad);

	box = create_box(ad);
	nf_it = elm_naviframe_item_push(ad->nf, "Thumbnail Util", NULL, NULL, box, NULL);
	elm_naviframe_item_pop_cb_set(nf_it, nf_it_pop_cb, ad);

	return ad->nf;
}

/*
 * @brief Function to create base gui structure
 * @param[in] ad The data structure to manage gui object
 */
static void create_base_gui(appdata_s *ad)
{
	Evas_Object *bg;

	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_conformant_set(ad->win, EINA_TRUE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	if (elm_win_wm_rotation_supported_get(ad->win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(ad->win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);

	/* Conformant */
	ad->conform = elm_conformant_add(ad->win);
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	/* Indicator BG */
	bg = elm_bg_add(ad->conform);
	elm_object_style_set(bg, "indicator/headerbg");
	elm_object_part_content_set(ad->conform, "elm.swallow.indicator_bg", bg);
	evas_object_show(bg);

	/* Naviframe */
	ad->nf = create_main_view(ad);
	elm_object_content_set(ad->conform, ad->nf);

	/* Show window after base gui is set up */
	evas_object_show(ad->win);
}

/*
 * @brief Hook to take necessary actions before main event loop starts
 * Initialize UI resources and application's data
 * If this function return true, the main loop of application starts
 * If this function return false, the application is terminated
 * @param[in] user_data The data to be passed to the callback function
 */
static bool app_create(void *user_data)
{
	/* Hook to take necessary actions before main event loop starts
	   Initialize UI resources and application's data
	   If this function returns true, the main loop of application starts
	   If this function returns false, the application is terminated */
	appdata_s *ad = user_data;

	create_base_gui(ad);

	return true;
}

/*
 * @brief This callback function is called when another application
 * sends the launch request to the application
 * @param[in] app_control The handle to the app_control
 * @param[in] user_data The data to be passed to the callback function
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/*
 * @brief This callback function is called each time
 * the application is completely obscured by another application
 * and becomes invisible to the user
 * @param[in] user_data The data to be passed to the callback function
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/*
 * @brief This callback function is called each time
 * the application becomes visible to the user
 * @param[in] user_data The data to be passed to the callback function
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/*
 * @brief This callback function is called once after the main loop of the application exits
 * @param[in] user_data The data to be passed to the callback function
 */
static void app_terminate(void *user_data)
{
	/* Release all resources. */
}

/*
 * @brief This function will be called when the language is changed
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

/*
 * @brief This function will be called when the orientation is changed
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

/*
 * @brief This function will be called when the region is changed
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

/*
 * @brief This function will be called when the battery is low
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

/*
 * @brief This function will be called when the memory is low
 * @param[in] event_info The system event information
 * @param[in] user_data The user data to be passed to the callback function
 */
static void ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

/*
 * @brief Main function of the application
 * @param[in] argc The argument count
 * @param[in] argv The argument vector
 */
int main(int argc, char *argv[])
{
	appdata_s ad = {0, };
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() is failed. err = %d", ret);

	return ret;
}
